﻿using Buddy.Wildstar.Game.Actors;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using Buddy.Wildstar.Game;


namespace Buddy.DefaultRoutine.Classes
{
	/*
Engineer Ability Breakdown		- (V = Volatility)

Mode: Provoke			- 60s CD		- Instant		- +10V/s for 10s (Defensive CD)
Mode: Eradicate			- 60s CD		- Instant		- +10V/s for 10s (DPS CD)

Obstruct Vision			- 30s CD		- Instant		- Interrupt + Blind
Zap						- 30s CD		- Instant		- Stun/Interrupt

Recursive Matrix		- 30s CD		- Instant		- Applies Defense/Stalwart around self for 10s (Defensive CD)
Shatter Impairment		- 20s CD		- Instant		- Removes all CCs and gives absorb [T4 - grants Empower, T8 - cleanses 2 debuffs]
Personal Defense Unit	- 45s CD		- Instant		- Grants Defense for it's duration, shield absorb when it dies [T4 causes it to last 60s instead of 30, T8 - restores health as well]
Unstable Anomaly		- 10s CD		- Instant		- Applies Wound

Urgent Withdrawl		- 1s CD			- Instant		- Snares targets [T4 gives Snare CC break, T8 grants CC immunity for 1.5s]

Artillerybot			- No CD[15s bot]- Instant		- Summons Pet, grants Barrage
Bruiserbot				- No CD[15s bot]- Instant		- Summons Pet, grants Blitz (taunt)
Diminisherbot			- No CD[15s bot]- Instant		- Just a damage/snare bot. [T4		- Grants 5% max health when active] (Tank bot)
Repairbot				- No CD[15s bot]- Instant		- Shield repair bot [T4 grants 15% shield mitigation] (Tank bot)

Quick Burst				- 8s CD			- Instant		- [Spell Proc] (T8 best used above 80V)
Feedback				- 8s CD			- Instant		- [Spell Proc]

Mortar Strike			- 10s CD		- Instant		- -25V
Electrocute				- No CD			- 3s Channel	- -8V/tick (0.5s tick) [T4 grants Empower (6 stacks total)]
Unsteady Miasma			- No CD			- Instant		- -40V,  Applies Blunder
Bolt Caster				- No CD			- Instant		- -25V
Particle Ejector		- No CD			- 3s Channel	- -5V/tick (0.5s tick) 135% threat
Thresher				- No CD			- Instant		- -50V

Volatile Injection		- 20s CD		- Instant		- +5V/tick (1s tick for 10s) [50V total] Applies Defense and Empower
Disruptive Module		- 12s CD		- 2s Cast		- +5V/tick (1s tick for 6s) [30V total] Restores shield to self +4 allies [T4 increases shield mitigation, T8 30-70V restores shield per tick]
Shock Pulse				- 10s CD		- 1.25s Cast	- +10V Applies Snare
Bio Shell				- 10s CD		- 1.9s Cast		- +30V (T4 instant cast between 30-70V) Applies Expose for 11s
Target Acquisition		- 10s CD		- 3s Channel	- +3V every 0.25s (per tick on the mob) [36V total] consumes applied "marks" at the end of channel
Ricochet				- 6s CD			- 1.25s Cast	- +20V, Applies Exhaust, +110% threat
Energy Auger			- 25s CD		- Instant	    - +0V
Flak Cannon				- No CD			- 2.5s Channel	- +5V per tick (0.5s tick) [25V total] +110% threat
Pulse Blast				- No CD			- 1.25s Cast	- +15V


Code Red				- 45s CD		- Instant		- AoE Taunt
Hyper Wave				- 15s CD		- Instant		- Taunt
	 */
	public class Engineer : BaseCombatClass
    {	
	MyGadget Gadget = new MyGadget();
	bool RecklessDashAMP = HasAMP("Reckless Dash");
	
        internal override async Task<bool> Pull(Actor target)
        {
				await PopFlask();
				return await Combat();


        }

		internal async Task<bool> TryCastBots()
		{
			// Do some pet management here...
			// Ensure the pets are actually in "Assist" mode. This is most useful for us.
			await EnsurePetStances();

			// Now keep up all the bots, and more or less just spam their abilities whenever we can.
			// NOTE: The spell tooltip in-game changes when you summon a pet. However, the ability internally doesn't change names.
			// So this will not only summon the bots, but also "spam" their abilities whenever they're off CD
			if (await SelfCast("Bruiserbot"))
				return true;
			if (await SelfCast("Artillerybot"))
				return true;
			if (await SelfCast("Diminisherbot"))
				return true;
			if (await SelfCast("Repairbot"))
				return true;
				
				
				
				
			return false;
		}

        internal override async Task<bool> Combat()
        {
			await ChugPotion();
		
			// Cast Interrupts
			//TODO - Obstruct Vision first if IA is detected on target
            if (Target != null && Target.IsCasting && Target.IsInCombat && await CastAny("Zap", "Obstruct Vision"))
                return true;

			// If we're in trouble, pop our defensive innate.
	        if (Me.HealthPercent < 30 && await SelfCast("Mode: Provoke"))
		        return true;

            // Shatter Impairment is a CC cleanse, and at T8 also a debuff cleanse
	        if ((Me.GetActiveCCs().Count > 0 || (IsSpellTier("Shatter Impairment", 8) && Me.Buffs.Any(b => b.IsHarmful))) && await SelfCast("Shatter Impairment"))
		        return true;

			// A defensive CD
	        if (Me.HealthPercent < 50 && await Cast("Recursive Matrix"))
		        return true;

			// Buffs/Debuffs
	        if (await CastAny("Personal Defense Unit", "Unstable Anomaly"))
		        return true;

			// T4 UW gives us a snare break
	        if (await Cast("Urgent Withdrawal"))
		        return true;
				
				
				
				
			//Uses a DPS gadget
			if ((Gadget.GetType() == 1) && await Gadget.UseGadget())
				return true;	

			// Pop our DPS innate.
	        if (Me.InnateResource <= 100 && await SelfCast("Mode: Eradicate"))
		        return true;				
				
				
			if (Me.HasBuff("Dealt Multi-Hit Damage") && Me.IsInCombat == true && (!IsSpellProcReady("Bolt Caster") && !IsSpellProcReady("Electrocute")) )	//Need to check if Casting or In GCD(Bolt Caster)
	        {//Since Quick Burst applies a GCD if we aren't already in one
				// QB T8 does a ton more damage at 80+ Volatility.
				// So lets include that logic.
		        if (IsSpellTier("Quick Burst", 8))
		        {
			        if (Me.InnateResource > 80 && await Cast("Quick Burst"))
				        return true;
		        }
		        else
		        {
					// Non-T8, just cast as soon as it's up
			        if (await Cast("Quick Burst"))
				        return true;
		        }
	        }	
				

			// TODO: Taunts!
			//if (await CastAny("Hyper Wave", "Code Red"))
			//	return true;


			
			//Using Hyper Wave as a filler attack for now (Off-GCD)
			if (await CastAny("Hyper Wave"))
		        return true;
			
			
			//Feedback also off-GCD free DPS
	        if (Me.HasBuff("Feedback") && await Cast("Feedback"))
		        return true;
			
			
			//Activates our Volatility Generator/Critical Buff first before attacking
			if ((!(CastingSpell("Target Acquisition")) //Prevents Target Acquisition from being interrupted. (Long Cast Time and also generates volatility)
			|| (!HasSpell("Target Acquisition"))) 
			&& await SelfCast("Volatile Injection"))
				return true;


				
			 

			
			// Increased priority for Tier 4 Bioshell and Ricochet, if you are in "The Zone" (30-70 volatility)
			// Checking for the buff (instead of IsSpellTier), makes the abilities slightly more responsive in case of rare mismatches resulting from
			// quickly going in and out of the 30-70 range. (Bio Shell and Ricochet Tier 4 buffs have a .5 second internal cooldown.)	
			// Added in additional checking for volatility generating buffs to not interrupt Electrocute when plenty of volatility is available. 
			if ((Me.HasBuff("Bio Shell") && Me.InnateResource <= 60) //60 Volatility now used in rare cases of misfires, and improved volatility management (was exceeding 100)
			&& (!((!IsSpellProcReady("Electrocute") || Me.InnateResource >= 49) && (Me.HasBuff("Volatile Injection", 3) || Me.HasBuff("Eradication ExoSuit", 2))) || !HasSpell("Electrocute") )
				&& await Cast("Bio Shell"))
				return true;

							
			if (Me.HasBuff("Ricochet") &&  await Cast("Ricochet"))
				return true;
				

				
				
			//Checks and saves a variable if we have the AMP.
	        if (RecklessDashAMP)
			{
				//Dash Range is 0-200 (2 tokens) 
				if (Me.DashesAvailable > 100 && Me.CanDash == true && Me.InnateResource <= 50
					//Doesn't interrupt casts
					&& ((IsSpellProcReady("Particle Ejector") || IsSpellProcReady("Electrocute") || Me.InnateResource <= 8) || (!HasSpell("Particle Ejector") && !HasSpell("Electrocute")))
					// Uses instant cast Bio Shell over dashing (For Bolt Caster Spec)
					&& ((Me.InnateResource < 30 || (HasSpell("Bio Shell") && !SpellManager.GetSpell("Bio Shell").IsSpellReady)) || (!HasSpell("Bio Shell")))
					)

		        {
			        GameManager.Movement.Dash(Buddy.Wildstar.Game.DashDirection.Forward);
			        return true;
		        }

	        }
	        // The following are all going to be "cast when high Volatility"
			// TODO: Engineer spenders are quite clunky. Only 1 has a cooldown. So you need to manage Volatility pretty well.
			
			// Mortar Strike has top priority due to cooldown
			// If Mortar Strike is Tier 4 or above, and the build has Target Acquisition, Target Acquisition gains increased priority to proc Overcharge
			//Otherwise if both the two abilities are missing (For Tier 4 Plus Mortar Strike), or it's less than tier 4, it will be used
			//with a volatility cost instead
			
			
			// NOTE IsSpellProcReady (not used for procs in this context) is used to prevent unnecessary spamming of abilities,
			// and instead used to increase the responsiveness of bot abilities (bottom of rotation)
			
			if ((IsSpellTier("Mortar Strike", 4) && HasSpell("Mortar Strike")) && await Cast("Target Acquisition"))
		        return true;	//High Priority for Target Acquisition, when using Mortar Strike Spec.
			if ((IsSpellTier("Mortar Strike", 4) && HasSpell("Target Acquisition")) && !IsSpellProcReady("Target Acquisition") && await Cast("Mortar Strike"))		
				{return true;}	//Now we have the MS Buff, we activate it
		
		
			if ((!IsSpellTier("Mortar Strike", 4) || !HasSpell("Target Acquisition")) && Me.InnateResource > 35 && await Cast("Mortar Strike")) //Uses ability normally
				return true;
	        if (Me.InnateResource > 80 && await Cast("Thresher"))
		        return true;
	        if (Me.InnateResource > 80 && await Cast("Unsteady Miasma"))
		        return true;
	        if (Me.InnateResource >= 38 && IsSpellProcReady("Electrocute") && await Cast("Electrocute"))
		        return true;
	        if (Me.InnateResource > 35 && IsSpellProcReady("Particle Ejector") && await Cast("Particle Ejector"))
		        return true;
	        if (Me.InnateResource > 35 && await Cast("Bolt Caster"))
		        return true;
				

				
				
			
					
				
				
			// These are all volatility builders, ordered by CD and "usefulness"
			// IsSpellProcReady prevents builders below from interrupting spenders. Overrides that however if neither are castable from very low volatility
			// Only applies this logic if either particle ejector or electrocute are slotted however.
	        if ((((IsSpellProcReady("Particle Ejector") || IsSpellProcReady("Electrocute")) || (Me.InnateResource >=0 && Me.InnateResource < 8)) 
			|| (!HasSpell("Particle Ejector") && !HasSpell("Electrocute"))) 
			&& await CastAny("Disruptive Module", "Shock Pulse", "Target Acquisition", "Energy Auger", "Ricochet", "Bio Shell", "Flak Cannon", "Pulse Blast"))
		       return true;			
						
				
			if (await TryCastBots())
		        return true;

				


            return false; 
        }

		private async Task EnsurePetStances()
		{
			foreach (var pet in GameManager.CurrentPets)
			{
				if (pet.Stance != PetStance.Assist)
				{
					GameManager.Lua.DoString("Pet_SetStance(0, 4)");
					// Only need to set it once. This changes it for all pets, so... yeah
					return;
				}
			}
		}
    }
}
