﻿using System.Linq;
using System.Threading.Tasks;
 
using Buddy.Wildstar.Game.Actors;
 
namespace Buddy.DefaultRoutine.Classes
{
		
        public class Medic : BaseCombatClass
        {
        MyGadget Gadget = new MyGadget();
                internal override async Task<bool> Heal()
                {
                        // Temporary disable of medic healing
                        // Will enable it again later at some point.
                        return false;
                        // Barrier (45s CD, 30m, "Targeted")
 
 
                        // Dispel 2 debuffs from self and 5 allies (15yds)
                        // Tier 4, free to cast if we dispel something. (Only CD is triggered)
                        // Tier 8, dispels 4 debuffs.
                        if (Me.Buffs.Count(b => b.IsHarmful) >= 1 && await SelfCast("Antidote"))
                                return true;
 
                        // Build actuators. Maybe do this further down? We only want to do this if we're in a group, or need a heal.
                        if (Me.InnateResource < 2 && await Cast("Emission"))
                                return true;
 
                        var partyHealthPercentage = PartyMembers.Average(s => s.HealthPercent);
 
                        // Drop Crisis Wave | 2 Actuators
                        if (partyHealthPercentage < 60 && await Cast("Crisis Wave"))
                                return true;
 
                        // Cast whenever it's up? Heals 4 allies and self. 15yd
                        if (partyHealthPercentage < 60 && IsSpellProcReady("Dual Shock") && await Cast("Dual Shock"))
                                return true;
 
                        // Mending probes is an HoT for self+4 allies for 12s. CD is also 12s. So cast whenever it's up.
                        // Side-effect at higher tiers is certain heals also "popping" their remaining heal.
                        if (partyHealthPercentage < 60 && await Cast("Mending Probes"))
                                return true;
                       
                        // Drop these here, since they aren't used above.
                        // Don't calculate stuff we don't want to worry about
                        var lowestHealthPercentage = PartyMembers.Min(s => s.HealthPercent);
                        var lowestShieldPercentage = PartyMembers.Min(s => s.ShieldPercent);
 
                        if (lowestHealthPercentage < 60 && lowestShieldPercentage < 60 && await Cast("Triage"))
                                return true;
 
                        // 0-17yd range. Heals 10 people, so make sure that we have at least 3 or 4 in our range.
                        if (partyHealthPercentage < 60 && await Cast("Flash"))
                                return true;
 
                        return false;
                }
 
                internal override async Task<bool> Pull(Actor target)
                {
                        // Note: medic pulls, are the same as medic combat.
                        // So just go through the normal combat logic to pull.
                        // TODO: Maybe pull with a long-range ability first?
						await PopFlask();
                        return await Combat();
                }
 
                internal override async Task<bool> Combat()
                {
					await ChugPotion();
				
				
                        // Interrupts
                        if (Target != null && Target.IsCasting && Target.IsInCombat && await CastAny("Paralytic Surge", "Magnetic Lockdown"))
                                return true;
 
						// TODO: Restrictor -> Cast when there's 3+ monsters standing in front of us. This will launch us backwards, and snare anything in front.
                        // Also useful for "backing up quckly" out of telegraphs
 
                        // Cleanses 2 beneficial buffs from 5 foes, as well as some damage.
                        // Tier 4 = 20% chance to build an actuator if purging a buff
                        // Tier 8 = Overload (shields deactivated on target)
                        if (Target != null && Target.Buffs.Count(b => b.IsBeneficial) > 1 && await Cast("Dematerialize"))
                                return true;
 
                        // Basically breaks us out of any CCs, restores shields, and pacifies enemies.
                        // Mostly going to be useful for CCs. But it's a 30s CD, so better logic may be needed.
                        if (Me.GetActiveCCs().Count > 1 && await SelfCast("Calm"))
                                return true;
 
                        if (Me.HealthPercent < 50 && Me.Buffs.Count(b => b.IsHarmful) >= 1 && await SelfCast("Antidote"))
                                return true;

						//High in the priority, as we want to be popping our abilities at the same time that Quantum Cascade is active.
						//IsSpellProcReady checks if there are enough actuators to use it
                        if (IsSpellProcReady("Quantum Cascade") && !CastingSpell("Quantum Cascade") && await Cast("Quantum Cascade"))
                            {
							return true;
							
							}
							
                        // If it's available, go ahead and cast it.
                        // 10s cooldown. 15yd range
                        // Don't cast it if we have full actuators. Since this builds 1. It's instant cast, so pop it whenever it's available under 4 actuators.
						// Want to use it Mid-Cast, as it procs a GCD.
                        if ((Me.InnateResource < 4 ||
							(!HasSpell("Gamma Rays") && !HasSpell("Quantum Cascade") && !HasSpell("Shield Surge"))) //ignores logic if we don't have a spender
						&& Me.HasBuff("Atomize")  && await Cast("Atomize"))
                                return true;
 
						if (await CastAny("Devastator Probes"))
                                return true;
						
						if (await Cast("Fissure"))
                           return true; 
						 
						
						//Higher Collider Priority if we're able to detonate probes.  
                        if (IsSpellTier("Devastator Probes", 4) && ((Me.InnateResource < 4 && IsSpellTier("Collider", 4)) || !IsSpellTier("Collider", 4)) && await Cast("Collider"))
                            return true; 						 
						 
						 
						 
                               
                        // Nullifier: 8s CD
                        // Devastator Probes: 12s CD
                        // Annihilation: 10s CD

						//Waits for the debuff to wear off before reapplying Nullifier to get the Tier 4 bonus damage
                        if (IsSpellTier("Nullifier", 4) && Target != null && !Target.HasBuff("Nullifier") && await Cast("Nullifier"))
								return true;
						//If nullifier is not Tier 4, spam it off cooldown.
                        if (!IsSpellTier("Nullifier", 4) && await Cast("Nullifier"))
								return true;	
								
								
								

						
						//Need to check if it's usable, as it now costs an actuator.
						//If we're already Casting Quantum Cascade (another spender), require us to have 2 or more actuators.						
                        if (((IsSpellProcReady("Annihilation") && !CastingSpell("Quantum Cascade")) ||  
							(Me.InnateResource > 1 && CastingSpell("Quantum Cascade")))
						&& await Cast("Annihilation"))
                                return true; 				
								

                        if (await CastAny("Urgency", "Restrictor"))
                                return true;
								
                        // Keep buffs up
                        // If we return Ok, then we cast the buff on ourselves.
                        if (await EnsureBuffs())
                                return true;   
 
                        // Spend actuators
                        // Gamma Rays = 2 actuators
                        // Quantum Cascade = 1 actuator per tick (0.5s per tick)
                        // Shield Surge = 2 actuator (no CD, 1.5s cast) 15yds - Deals damage to 5 foes, and/or heals a bunch of shield + buffs depending on spell tier
                        if (IsSpellProcReady("Gamma Rays") && await Cast("Gamma Rays"))
                                return true; //Checking by resource is too slow due to Actuator refund mechanics, therefore we can check if it's usable instead.

                        if (Me.InnateResource >= 2 && await Cast("Shield Surge"))
                                return true;
 
                        // Tier 8 Magnetic Lockdown builds 2 actuators.
                        // Outside of that, we probably only want to use it when it's up for the sake of root+snare and a bit of damage + interrupt.
                        if (Me.InnateResource <= 2 && IsSpellTier("Magnetic Lockdown", 8) && await Cast("Magnetic Lockdown"))
                                return true;
 
                        // If the tier is 4+, builds an actuator, so we wait to lose actuators first.
						//Tier 3 and below, we ignore that and cast it normally.
                        if (((Me.InnateResource < 4 && IsSpellTier("Collider", 4)) || !IsSpellTier("Collider", 4)) && await Cast("Collider"))
                                return true; 
										
 
                        // Build actuators
						//Checks if Gamma Rays is not usable before activating Discharge, ignores if we do not have Gamma Rays
						//If we have Quantum Cascade, we want to use discharge only when Resources are 0, or when mid-cast of Quantum Cascade.
						//This will ensure the bot waits to use Quantum Cascade again, instead of instantly spending the GCD on Discarge.
                        if ( (Me.InnateResource < 2 || !HasSpell("Gamma Rays")) && 
							(Me.InnateResource < 1 || !HasSpell("Quantum Cascade") || CastingSpell("Quantum Cascade")) &&
						
						await CastAny("Discharge", "Emission"))
                                return true;
	
                               
 
						return false;
                        //return await Cast("Recharge");
                }
 
                private async Task<bool> EnsureBuffs()
                {
                        //Uses a DPS gadget
                        if ((Gadget.GetType() == 1) && await Gadget.UseGadget())
                                return true;   
               
                        // Pop our innate whenever we have low health, or no innate resources.
                        if ((Me.HealthPercent < 30 || Me.InnateResource <= 1) && (!(CastingSpell("Gamma Rays"))) && !CastingSpell("Discharge") && await SelfCast("Energize"))
                                return true;
 
                        // TODO: Ensure that we won't fling ourselves off a cliff, etc.
                        if (Me.IsRooted() && await SelfCast("Urgency"))
                                return true;
 
                        if (!Me.HasBuff("Defense") && await SelfCast("Protection Probes"))
                                return true;
                               
                        if (!Me.HasBuff("Empowering Probes") && (!(CastingSpell("Discharge")) //Prevents Discharge from being interruptted.
                        || (!HasSpell("Discharge"))) && await SelfCast("Empowering Probes"))
                                return true;
 
 
 
                        return false;
                }
        }
}