﻿using Buddy.Wildstar.Game.Actors;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using Buddy.Wildstar.Game;

using CCStateType = Buddy.Wildstar.Game.CCStateType;

namespace Buddy.DefaultRoutine.Classes
{
    public class Warrior : BaseCombatClass
    {
		MyGadget Gadget = new MyGadget();
		bool SpeedBurstAMP = HasAMP("Speed Burst");
		/*
		 Warrior ability breakdown (KE = Kinetic Energy)
		 * 
		 * Relentless Strikes		- No CD 				- +180 KE
		 * Menacing Strike			- No CD  				- +300 KE [Filler Ability, +150% threat]
		 * Ripsaw					- 10s CD 				- No KE
		 * Savage Strikes			- 15s CD				- +125 KE (per hit [hits twice])
		 * Augmented Blade			- 15s CD				- No KE (applies Wound to next hit)
		 * Smackdown				- 10s CD				- No KE (applies Expose [9s], T8 applies Blunder [3s])
		 * Tremor					- 12s CD 				- No KE (1.5m pushback [interrupt?] to 10 monsters. Nearest gets damage)
		 * Breaching Strikes		- 10s CD 				- +125 KE [Spell Proc]
		 * Jolt						- 12s CD 				- +250 KE [+150% threat]
		 * Atomic Spear				- 10s CD 				- +250 KE [Spell Proc, +200% threat]
		 * Polarity Field			- 15s CD 				- No KE [Applies Exhaust, +150% threat] 10s field, 1s tick (T4 creates it around 2 mobs)
		 * Power Link				- 25s CD 				- No KE [Applies Empower to self + 4 party members for 10s]
		 * Thether Bolt				- 30s CD				- No KE [Applies Tether + minimal damage, T8 - tether 2 mobs]
		 * 
		 * 
		 * Rampage					- 8s CD					- -250 KE (4 charges)
		 * Whirlwind				- 8s CD (2s channel)	- -250 KE/tick (4 ticks at 0.5s intervals)
		 * Plasma Wall				- 8s CD (2s channel)	- -250 KE/tick (4 ticks at 0.5s intervals) [Very large damage reduction + some damage]
		 * Bolstering Strike		- 8s CD					- -250 KE [Restores Shields, T8 restores shield to 4 allies within 7m]
		 * Shield Burst				- 6s CD					- -250 KE (+150% threat, restore shield if used within 3s of Bolstering Strike or Emergency Reserves)
		 * 
		 * Bum Rush					- 12s CD				- No KE (Applies Defense for 3s)
		 * 
		 * 
		 * Leap						- 15s CD				- +250 KE (Removes roots, damage if hits [use for pulling])
		 * 
		 * Expulsion				- 8s CD					- No KE (Removes 2 debuffs [T8 clears 3 debuffs]) + minor damage
		 * 
		 * Atomic Surge				- 20s CD				- No KE (Taunt)
		 * Plasma Blast				- 15s CD				- No KE (AoE Taunt, reduces taunted mobs damage by 25% for 5s)
		 * 
		 * Kick						- 20s CD				- No KE (Interrupt, T4 reduces CD by 2s)
		 * Flash Bang				- 25s CD				- No KE (AoE Interrupt)
		 * Grapple					- 20s CD				- No KE (Interrupt + Root to 2 mobs)
		 * 
		 * Emergency Reserves		- 25s CD				- No KE (Restores shield, 100% shield mitigation for 6s, and N interrupt armor for 5s) [T8 grants the buff to 4 party members within 20m]
		 * Sentinel					- 10s CD				- No KE [Applies Guard to lowest health party member for 18s]
		 * Defense Grid				- 30s CD				- No KE [Applies Defense to party members in the field]
		 * 
		 * Unstoppable Force		- 30s CD				- No KE [Removes all CC effects, applies Freedom (immune to CC) for 3.2s
		 
		 */

		internal float KineticEnergy { get { return GameManager.LocalPlayer.InnateResource; } }

		internal override async Task<bool> Heal()
		{
			var hPercent = Me.HealthPercent;

			await ChugPotion();	
			
			if (hPercent < 30 && await Cast("Emergency Reserves"))
				return true;

			if (hPercent < 70 && await Cast("Bum Rush"))
				return true;
			if (hPercent < 70 && await Cast("Bolstering Strike"))
				return true;
			if (hPercent < 70 && await Cast("Menacing Strike"))
				return true;

			return false;
        }

        internal override async Task<bool> Pull(Actor target)
        {
			await PopFlask();
			// LEAP INTO COMBAT LIKE THE MIGHTY THOR!
	        if (await Cast("Leap"))
		        return true;

            return await Combat(); 
        }

        internal override async Task<bool> Combat()
        {
            // Interrupt
			// Tremor has the lowest CD. (Not sure if it's a proper interrupt, but it may be)
			// Kick -> Grapple are both interrupts to a single target [Grapple roots 2]
			// Flash Bang is an AoE interrupt (last resort)
            if (Target.IsCasting && Target.IsInCombat && await CastAny("Kick", "Grapple", "Flash Bang"))
                return true;
				
				
			if (Me.HasBuff("Breaching Strikes") && await Cast("Breaching Strikes"))
		        return true;
			
			if (Me.InnateResource >= 500 && await Cast("Rampage"))
		        return true;
				
			if (await CastAny("Tremor"))
		        return true;
			
			if (await CastAny("Whirlwind"))
		        return true;
			
			
			if (await CastAny("Savage Strike","Relentless Strikes", "Jolt", "Ripsaw", "Menacing Strike"))
		        return true;
			
			

			// So these are procs, but we still want to cast them after Augmented Blade
			// They are raw-damage abilities
		
				
	        if (IsSpellProcReady("Atomic Spear") && await Cast("Atomic Spear"))
		        return true;

			// Start generating some KE
			// Only Ripsaw here doesn't generate any KE. But we'll use it as one, since it's a pure-damage ability.
	        if (await CastAny("Savage Strike", "Jolt", "Ripsaw", "Menacing Strike", "Relentless Strikes"))
		        return true;

			// Break out of any CCs
	        if (Me.GetActiveCCs().Count > 0 && await Cast("Unstoppable Force"))
		        return true;

			// KILL ALL THE THINGS!
	        if (Me.HealthPercent > 75 && await Cast("Stance: Onslaught"))
		        return true;

			// RUN AWAY FROM ALL THE THINGS!
	        if (Me.HealthPercent < 25 && await Cast("Stance: Juggernaut"))
		        return true;
	
				
			//Uses a DPS gadget
            if ((Gadget.GetType() == 1) && await Gadget.UseGadget())
                return true;  
			// Restore some shield, and give some interrupt armor + shield mitigation
			// TODO: This is also a good way to prevent our channeled abilities from being broken [gives 2 interrupt armor]
	        if (Me.HealthPercent < 30 && await SelfCast("Emergency Reserves"))
		        return true;

			// Defense is applied by both of these. So first lets try Defense Grid [since that applies for party members as well]
			// Then revert to Bum Rush
	        if (!Me.HasBuff("Defense") && await CastAny("Defense Grid", "Bum Rush"))
		        return true;
	        if (!Me.HasBuff("Guard") && await SelfCast("Sentinel"))
		        return true;

			// These are all CD abilities
			// Ordered by importance to what we're doing. These all apply some form of debuff, or self-buff.
			// We don't want to waste an Augmented Blade on a low-damage buff-applying ability if at all possible.
	        if (await CastAny("Smackdown", "Polarity Field", "Tether Bolt", "Power Link", "Augmented Blade"))
		        return true;

			// Spend some KE please :)
			// These are all on cooldowns, and ordered by their usefulness.
			// The first 3 are instant cast, and the rest all have cast/channel times associated.

			
			//Checking for correct Amp
			if (SpeedBurstAMP)
			{
			
			if (Me.DashesAvailable > 100 && Me.CanDash == true  && Me.IsInCombat == true )//Dash Range is 0-200 (2 tokens) 
																																
				{GameManager.Movement.Dash(Buddy.Wildstar.Game.DashDirection.Forward);
				return true;}
				
			}


			// TODO: Taunts
			//if (await CastAny("Atomic Surge @@@@Single target Taunt@@@@", "Plasma Blast @@@@AOE Taunt@@@@"))
			//	return true;

            return false;
        }

    }
}
